package com.socklabs.whoisthat.configuration;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.web.FilterChainProxy;
import org.springframework.web.servlet.view.InternalResourceViewResolver;
import org.springframework.web.servlet.view.UrlBasedViewResolver;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;
import org.springframework.web.servlet.view.freemarker.FreeMarkerViewResolver;

/**
 * A configuration class for several misc spring beans. Some of these could probably be moved to the WebMvcConfig
 * configuration class.
 * @author ngerakines
 * @since 2/19/12
 */
@Configuration
@ComponentScan( basePackages = "com.socklabs.whoisthat" )
public class SpringConfig {

	@Bean(name = "springSecurityFilterChain")
	public FilterChainProxy filterChainProxy() {
		FilterChainProxy filterChainProxy = new FilterChainProxy();
		return filterChainProxy;
	}

	@Bean(name = "viewResolver")
	public FreeMarkerViewResolver freeMarkerViewResolver() {
		FreeMarkerViewResolver freeMarkerViewResolver = new FreeMarkerViewResolver();
		freeMarkerViewResolver.setCache(true);
		freeMarkerViewResolver.setContentType("text/html; charset=UTF-8");
		freeMarkerViewResolver.setPrefix("/");
		freeMarkerViewResolver.setSuffix(".ftl");
		freeMarkerViewResolver.setExposeSpringMacroHelpers(true);
		freeMarkerViewResolver.setRequestContextAttribute("rc");
		freeMarkerViewResolver.setOrder(1);
		return freeMarkerViewResolver;
	}

	@Bean
	public UrlBasedViewResolver internalResourceViewResolver() {
		InternalResourceViewResolver internalResourceViewResolver = new InternalResourceViewResolver();
		internalResourceViewResolver.setOrder(2);
		return internalResourceViewResolver;
	}

	@Bean
	public FreeMarkerConfigurer freeMarkerConfigurer() {
		FreeMarkerConfigurer freeMarkerConfigurer = new FreeMarkerConfigurer();
		freeMarkerConfigurer.setTemplateLoaderPaths(new String[]{"/WEB-INF/ftl/"});
		freeMarkerConfigurer.setDefaultEncoding("UTF-8");
		return freeMarkerConfigurer;
	}

}
