package com.socklabs.whoisthat.configuration;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseBuilder;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseType;

import javax.sql.DataSource;

/**
 * A configuration class that provides a local/in-memory db using H2SQL. This configuration class is used when the
 * "embedded" profile is active.
 * @author ngerakines
 * @since 2/19/12
 */
@Configuration
@Profile("embedded")
public class StandaloneDataConfig {

	@Bean
	public DataSource dataSource() {
		return new EmbeddedDatabaseBuilder()
				.setType(EmbeddedDatabaseType.HSQL)
				.addScript("classpath:schema.sql")
				.addScript("classpath:embedded.sql")
				.build();
	}

}
