package com.socklabs.whoisthat.mvc;

import com.socklabs.whoisthat.mvc.model.Account;
import com.socklabs.whoisthat.mvc.model.AccountMeta;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;
import java.util.UUID;

/**
 * @author ngerakines
 * @since 2/19/12
 */
public class AccountManagerImpl implements AccountManager {

	@Autowired
	private JdbcTemplate jdbcTemplate;

	@Override
	public Account getByEmail(final String email) {
		List<Account> accounts = jdbcTemplate.query("SELECT * FROM account WHERE email = ?", new Object[]{ email }, accountRowMapper);
		if (accounts.size() == 1) {
			Account account = accounts.get(0);
			for (AccountMeta accountMeta : getAccountMetaByEmail(email)) {
				account.addMeta(accountMeta);
			}
			return account;
		}
		return null;
	}

	@Override
	public void create(final String email, final String password) {
		final String SQL = "insert into account values (?, ?, CURRENT_TIMESTAMP, CURRENT_TIMESTAMP, null, null)";
		jdbcTemplate.update(SQL, email, password);
	}

	@Override
	public String addMeta(String email, String label, String value) {
		final String SQL = "insert into account_meta values (?, ?, ?, ?, CURRENT_TIMESTAMP)";
		final String id = UUID.randomUUID().toString();
		jdbcTemplate.update(SQL, id, email, label, value);
		return id;
	}

	@Override
	public List<AccountMeta> getAccountMetaByEmail(final String email) {
		return jdbcTemplate.query("SELECT * FROM account_meta WHERE email = ?", new Object[]{ email }, accountMetaRowMapper);
	}

	private static final RowMapper<Account> accountRowMapper = new RowMapper<Account>() {
		@Override
		public Account mapRow(ResultSet resultSet, int i) throws SQLException {
			/* created_at TIMESTAMP, updated_at TIMESTAMP, last_login_at TIMESTAMP, deleted_at TIMESTAMP */
			Timestamp createdAt = resultSet.getTimestamp("created_at");
			Timestamp updatedAt = resultSet.getTimestamp("updated_at");
			Timestamp lastLoginAt = resultSet.getTimestamp("last_login_at");
			Timestamp deletedAt = resultSet.getTimestamp("deleted_at");
			return new Account( /* email varchar(255), name varchar(255), password varchar(255) */
				resultSet.getString("email"),
				resultSet.getString("password"),
				createdAt != null ? new DateTime(createdAt.getTime()) : null,
				updatedAt != null ? new DateTime(updatedAt.getTime()) : null,
				lastLoginAt != null ? new DateTime(lastLoginAt.getTime()) : null,
				deletedAt != null ? new DateTime(deletedAt.getTime()) : null
			);
		}
	};

	private static final RowMapper<AccountMeta> accountMetaRowMapper = new RowMapper<AccountMeta>() {
		@Override
		public AccountMeta mapRow(ResultSet resultSet, int i) throws SQLException {
			Timestamp createdAt = resultSet.getTimestamp("created_at");
			return new AccountMeta( /* id varchar(255), email varchar(255), label varchar(255), note clob */
				resultSet.getString("id"),
				resultSet.getString("email"),
				resultSet.getString("label"),
				resultSet.getString("note"),
				createdAt != null ? new DateTime(createdAt.getTime()) : null
			);
		}
	};

}