package com.socklabs.whoisthat.mvc.controller;

import com.socklabs.whoisthat.mvc.AliasValidator;
import com.socklabs.whoisthat.mvc.EntityManager;
import com.socklabs.whoisthat.mvc.NoteValidator;
import com.socklabs.whoisthat.mvc.model.AliasForm;
import com.socklabs.whoisthat.mvc.model.NoteForm;
import com.socklabs.whoisthat.mvc.model.Tuple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * Controller for alias CRUD.
 *
 * @author ngerakines
 * @since 2/25/12
 */
@Controller
public class AliasController {

	private final static Logger logger = LoggerFactory.getLogger(AliasController.class);

	@Autowired
	private EntityManager entityManager;

	@InitBinder
	protected void initBinder(WebDataBinder binder) {
		binder.setValidator(new AliasValidator());
	}

	@RequestMapping(value = "/alias", method = RequestMethod.POST)
	public String addNote(@Validated AliasForm aliasForm) throws Exception {
		logger.info("/alias called with alias form {}", aliasForm);

		List<Tuple<String,String>> groups = entityManager.getGroupsForEntities(aliasForm.getEntity(), aliasForm.getAlias());

		Set<String> groupIds = new HashSet<String>();
		for (Tuple<String,String> group : groups) {
			groupIds.add(group.getValueA());
		}
		if (groupIds.size() > 1) {
			throw new Exception("Merge group feature not implemented.");
		}

		String groupId = UUID.randomUUID().toString();
		if (groupIds.size() > 0) {
			groupId = groupIds.iterator().next();
		}

		entityManager.setGroup(groupId, aliasForm.getAlias());
		entityManager.setGroup(groupId, aliasForm.getEntity());

		// get group for entries
		// if no group for either, generate group key
		// for each entry without a group, create group record for it
		return "redirect:/dashboard";
	}

}
