package com.socklabs.whoisthat.mvc.controller;

import com.socklabs.whoisthat.mvc.AccountManager;
import com.socklabs.whoisthat.mvc.EntityManager;
import com.socklabs.whoisthat.mvc.MvcUtil;
import com.socklabs.whoisthat.mvc.model.Account;
import com.socklabs.whoisthat.mvc.model.AccountMeta;
import com.socklabs.whoisthat.mvc.model.Entity;
import com.socklabs.whoisthat.mvc.model.Tripple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * The controller for the user dashboard.
 *
 * This controller is protected by login using spring security rules.
 *
 * @author ngerakines
 * @since 2/19/12
 */
@Controller
public class DashboardController {

	private static final Logger logger = LoggerFactory.getLogger(DashboardController.class);

	@Autowired
	private AccountManager accountManager;

	@Autowired
	private EntityManager entityManager;

	@RequestMapping(value = "/dashboard", method = RequestMethod.GET)
	public String getIndex(
			HttpServletRequest httpServletRequest,
			ModelMap modelMap,
			@RequestParam(value = "offset", required = false, defaultValue = "0") Integer offset,
			@RequestParam(value = "count", required = false, defaultValue = "25") Integer count) {

		serverDashboard(httpServletRequest, modelMap, null, offset, count);

		return "dashboard";
	}

	@RequestMapping(value = "/dashboard/{source}", method = RequestMethod.GET)
	public String getSource(
			HttpServletRequest httpServletRequest,
			ModelMap modelMap,
			@PathVariable("source") String source,
			@RequestParam(value = "offset", required = false, defaultValue = "0") Integer offset,
			@RequestParam(value = "count", required = false, defaultValue = "25") Integer count) {

		serverDashboard(httpServletRequest, modelMap, source, offset, count);

		return "dashboard";
	}

	private void serverDashboard(HttpServletRequest httpServletRequest, ModelMap modelMap, String source, Integer offset, Integer count) {
		User user = MvcUtil.getPrincipal(httpServletRequest.getSession());
		logger.info("Loading dashboard for {}", user);

		Account account = accountManager.getByEmail(user.getUsername());
		logger.info("account {}", account);

		if (count > 25) {
			count = 25;
		}
		if (offset < 0) {
			offset = 0;
		}

		List<Tripple<String,String,String>> allEntities = entityManager.getAllNamesByAccount(user.getUsername());
		modelMap.addAttribute("allEntities", allEntities);

		List<Entity> entities = new ArrayList<Entity>();

		if (source == null) {
			entities.addAll(entityManager.getByAccountPaginated(user.getUsername(), offset, count));
		} else {
			entities.addAll(entityManager.getByAccountAndSourcePaginated(user.getUsername(), source, offset, count));
		}

		modelMap.addAttribute("entities", entities);
		modelMap.addAttribute("lastPage", offset - count);
		modelMap.addAttribute("nextPage", offset + count);

		modelMap.addAttribute("dashboardFilter", "all");

		List<AccountMeta> accountMetaList = account.getAccountMetaList();

		Map<String, Integer> counts = getCounts(accountMetaList);
		modelMap.addAttribute("counts", counts);
	}

	private Map<String, Integer> getCounts(final List<AccountMeta> accountMetaList) {
		final Map<String, Integer> counts = getCountsDefault();
		for (String service : counts.keySet()) {
			for (AccountMeta accountMeta : accountMetaList) {
				if (accountMeta.getLabel().equals(service + "/count")) {
					counts.put(service, Integer.valueOf(accountMeta.getValue()));
				}
			}
		}
		return counts;
	}

	private Map<String, Integer> getCountsDefault() {
		Map<String, Integer> defaults = new HashMap<String, Integer>();
		defaults.put("user", 0);
		defaults.put("facebook", 0);
		defaults.put("twitter", 0);
		defaults.put("github", 0);
		defaults.put("foursquare", 0);
		return defaults;
	}
}
