/*
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.socklabs.whoisthat.mvc.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.social.connect.Connection;
import org.springframework.social.connect.ConnectionRepository;
import org.springframework.social.facebook.api.Facebook;
import org.springframework.social.facebook.api.FacebookProfile;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.inject.Inject;
import java.util.List;

/**
 * A controller for Facebook connection configuration. If accessed without a valid Facebook connection it will redirect
 * the user to the connect/status page. If a connection is present, it will (should) present Facebook specific
 * configuration and actions.
 *
 * @todo Consider merging this page with the /dashboard/facebook page.
 * @author ngerakines
 * @since 2/19/2012
 */
@Controller
public class FacebookProfileController {

	private static final Logger logger = LoggerFactory.getLogger(FacebookProfileController.class);

	@Inject
	private ConnectionRepository connectionRepository;

	@RequestMapping(value="/facebook", method= RequestMethod.GET)
	public String home(Model model) {
		Connection<Facebook> connection = connectionRepository.findPrimaryConnection(Facebook.class);
		if (connection == null) {
			return "redirect:/connect/facebook";
		}
		FacebookProfile facebookProfile = connection.getApi().userOperations().getUserProfile();
		model.addAttribute("profile", facebookProfile);
		return "facebook-profile";
	}

}
