package com.socklabs.whoisthat.mvc.model;

import org.joda.time.DateTime;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: ngerakines
 * Date: 2/19/12
 * Time: 8:43 PM
 * To change this template use File | Settings | File Templates.
 */
public class Entity {
	/*
	create table entity (id varchar(255), account varchar(255), source varchar(255), label varchar(255));
	 */

	private String id;
	private String account;
	private String source;
	private String label;

	private String group;

	private DateTime createdAt;
	private DateTime lastUpdatedAt;
	private DateTime orphanedAt;
	private DateTime deletedAt;

	private List<String> tags;

	private List<EntityMeta> metas;

	public Entity(String id, String account, String source, String label, DateTime createdAt, DateTime lastUpdatedAt, DateTime orphanedAt, DateTime deletedAt) {
		this.id = id;
		this.account = account;
		this.source = source;
		this.label = label;
		this.createdAt = createdAt;
		this.lastUpdatedAt = lastUpdatedAt;
		this.orphanedAt = orphanedAt;
		this.deletedAt = deletedAt;
		this.metas = new ArrayList<EntityMeta>();
		this.tags = new ArrayList<String>();
	}

	public String getId() {
		return id;
	}

	public String getAccount() {
		return account;
	}

	public String getSource() {
		return source;
	}

	public String getLabel() {
		return label;
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public List<EntityMeta> getMetas() {
		return metas;
	}

	public void setMetas(List<EntityMeta> metas) {
		this.metas = metas;
	}

	public void addMetas(final EntityMeta entityMeta) {
		metas.add(entityMeta);
	}

	public List<String> getTags() {
		return tags;
	}

	public void setTags(List<String> tags) {
		this.tags = tags;
	}

	public void addTags(final String tag) {
		tags.add(tag);
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;

		Entity entity = (Entity) o;

		if (account != null ? !account.equals(entity.account) : entity.account != null) return false;
		if (id != null ? !id.equals(entity.id) : entity.id != null) return false;
		if (label != null ? !label.equals(entity.label) : entity.label != null) return false;
		if (metas != null ? !metas.equals(entity.metas) : entity.metas != null) return false;
		if (tags != null ? !tags.equals(entity.tags) : entity.tags != null) return false;
		if (source != null ? !source.equals(entity.source) : entity.source != null) return false;

		return true;
	}

	@Override
	public int hashCode() {
		int result = id != null ? id.hashCode() : 0;
		result = 31 * result + (account != null ? account.hashCode() : 0);
		result = 31 * result + (source != null ? source.hashCode() : 0);
		result = 31 * result + (label != null ? label.hashCode() : 0);
		result = 31 * result + (metas != null ? metas.hashCode() : 0);
		result = 31 * result + (tags != null ? tags.hashCode() : 0);
		return result;
	}

	@Override
	public String toString() {
		return "Entity{" +
				"id='" + id + '\'' +
				", account='" + account + '\'' +
				", source='" + source + '\'' +
				", label='" + label + '\'' +
				", metas=" + metas +
				", tags=" + tags +
				'}';
	}

	public String getDisplayName() {
		if (metas != null) {
			for (EntityMeta entityMeta : metas) {
				if (entityMeta.getLabel().equals("label")) {
					return entityMeta.getValue();
				}
			}
		}
		return label;
	}

	public String getNote() {
		if (metas != null) {
			for (EntityMeta entityMeta : metas) {
				if (entityMeta.getLabel().equals("note")) {
					return entityMeta.getValue();
				}
			}
		}
		return "";
	}

}
