package com.socklabs.whoisthat;

import com.socklabs.whoisthat.configuration.*;
import com.socklabs.whoisthat.mvc.AccountManager;
import com.socklabs.whoisthat.mvc.model.Account;
import com.socklabs.whoisthat.mvc.model.AccountMeta;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.List;

import static junit.framework.Assert.*;

/**
 * Created by IntelliJ IDEA.
 * User: ngerakines
 * Date: 2/20/12
 * Time: 8:58 AM
 * To change this template use File | Settings | File Templates.
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {AppConfig.class, DataManagerConfig.class, SecurityConfig.class, StandaloneDataConfig.class})
@ActiveProfiles({"embedded", "test"})
public class AccountManagerTest {

	@Autowired
	private AccountManager accountManager;

	@Test
	public void lookupAccount() {
		Account account = accountManager.getByEmail("nick@gerakines.net");
		assertNotNull(account);
		assertEquals(account.getEmail(), "nick@gerakines.net");
	}

	@Test
	public void lookupInvalidAccount() {
		Account account = accountManager.getByEmail("user@site");
		assertNull(account);
	}

	@Test
	public void createMetas() {
		Account account = accountManager.getByEmail("nick@gerakines.net");
		String accountMetaId = accountManager.addMeta(account.getEmail(), "settings/twitter/fetchFollowersAsFriends", "1");
		assertNotNull(accountMetaId);
		List<AccountMeta> accountMetaList = accountManager.getAccountMetaByEmail(account.getEmail());
		assertEquals(accountMetaList.size(), 2);

		for (AccountMeta accountMeta : accountMetaList) {
			if (accountMeta.getId().equals(accountMetaId)) {
				assertEquals(accountMeta.getId(), accountMetaId);
				assertEquals(accountMeta.getAccountId(), account.getEmail());
				assertEquals(accountMeta.getLabel(), "settings/twitter/fetchFollowersAsFriends");
				assertEquals(accountMeta.getValue(), "1");
				return;
			}
		}
		fail("Dit not encounter account meta.");
	}

}
