package com.socklabs.whoisthat;

import com.socklabs.whoisthat.configuration.AppConfig;
import com.socklabs.whoisthat.configuration.DataManagerConfig;
import com.socklabs.whoisthat.configuration.SecurityConfig;
import com.socklabs.whoisthat.configuration.StandaloneDataConfig;
import com.socklabs.whoisthat.mvc.AccountManager;
import com.socklabs.whoisthat.mvc.EntityManager;
import com.socklabs.whoisthat.mvc.model.Entity;
import com.socklabs.whoisthat.mvc.model.EntityMeta;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;

/**
 * Created by IntelliJ IDEA.
 * User: ngerakines
 * Date: 2/20/12
 * Time: 9:45 AM
 * To change this template use File | Settings | File Templates.
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {AppConfig.class, DataManagerConfig.class, SecurityConfig.class, StandaloneDataConfig.class})
@ActiveProfiles({"embedded", "test"})
public class EntityManagerTest {

	private final Logger logger = LoggerFactory.getLogger(EntityManagerTest.class);

	@Autowired
	private AccountManager accountManager;

	@Autowired
	private EntityManager entityManager;

	@Test
	public void loadEntity() {
		List<Entity> entity = entityManager.getByAccount("nick@gerakines.net");
		assertEquals(entity.size(), 3);
	}

	@Test
	public void tags() {
		List<Entity> entities = entityManager.getByAccount("nick@gerakines.net");
		logger.info("entities {}", entities);
		for (Entity entity : entities) {
			if (entity.getLabel().equals("Carolyn Gerakines")) {
				assertEquals(entity.getAccount(), "nick@gerakines.net");
				assertEquals(entity.getDisplayName(), "Carolyn Gerakines");
				assertEquals(entity.getTags().size(), 2);
				List<String> tags = entity.getTags();
				Collections.sort(tags);
				assertEquals(tags.get(0), "family");
				assertEquals(tags.get(1), "wife");
				return;
			}
		}
		fail("Did not encounter entity.");
	}

	@Test
	public void createEntity() {
		String entityId = entityManager.create("nick@gerakines.net", "user", "Sharon Armstrong");
		for (Entity entity : entityManager.getByAccount("nick@gerakines.net")) {
			if (entity.getLabel().equals("Sharon Armstrong")) {
				assertEquals(entity.getId(), entityId);
				assertEquals(entity.getAccount(), "nick@gerakines.net");
				assertEquals(entity.getDisplayName(), "Sharon Armstrong");
				assertEquals(entity.getTags().size(), 0);
				return;
			}
		}
		fail("Did not encounter entity.");
	}

	@Test
	public void createEntityMeta() {
		String entityId = entityManager.create("nick@gerakines.net", "user", "Michael Armstrong");
		entityManager.addMeta(entityId, "label", "Mike Armstrong");
		for (Entity entity : entityManager.getByAccount("nick@gerakines.net")) {
			if (entity.getLabel().equals("Michael Armstrong")) {
				assertEquals(entity.getId(), entityId);
				assertEquals(entity.getAccount(), "nick@gerakines.net");
				assertEquals(entity.getDisplayName(), "Mike Armstrong");
				assertEquals(entity.getTags().size(), 0);

				List<EntityMeta> entityMetas = entity.getMetas();
				assertEquals(entityMetas.size(), 1);
				assertEquals(entityMetas.get(0).getEntityId(),  entityId);
				assertEquals(entityMetas.get(0).getLabel(), "label");
				assertEquals(entityMetas.get(0).getValue(), "Mike Armstrong");
				return;
			}
		}
		fail("Did not encounter entity.");
	}

	@Test
	public void paginatedEntries() {
		List<String> uniqueIds = new ArrayList<String>();
		for (Entity entity : entityManager.getByAccountPaginated("nick@gerakines.net", 0, 3)) {
			uniqueIds.add(entity.getLabel());
		}
		for (Entity entity : entityManager.getByAccountPaginated("nick@gerakines.net", 3, 3)) {
			uniqueIds.add(entity.getLabel());
		}
		assertEquals(uniqueIds.size(), 5);
		logger.info("uniqueIds {}", uniqueIds);
	}

	@Test
	public void createAlias() {
		String entityIdA = entityManager.create("nick@gerakines.net", "user", "Michael Armstrong");
		String entityIdB = entityManager.create("nick@gerakines.net", "user", "Mike Armstrong");
		entityManager.setGroup("themikes", entityIdA);
		entityManager.setGroup("themikes", entityIdB);
	}
}
